var util   = require('util'),
    sinon  = require('sinon'),
    should = require('should'),
    http   = require('http'),
    needle = require('./../'),
    EventEmitter = require('events').EventEmitter;

var FakeRequest = function(args, callback){
  EventEmitter.apply(this);
  this.end = function(){ /* noop */ };
  if (callback) callback();
}

describe('options', function(){

  var req, spy;

  var disableRequests = function(){
    spy = sinon.stub(http, 'request');
    // spyOn(http, 'request').andReturn(new FakeRequest());
    // spyOn(this, 'setTimeout').andReturn(true); // dont wait for default timeout
  }

  describe('module', function(){

    it('should expose version', function(){
      should.exist(needle.version);
      needle.version.should.be.a('string');
    });

    it('should export head/get/post/put/delete methods', function(){
      Object.keys(needle).join(' ').should.equal('version head get post put delete');
    })

  })

  describe('requests', function(){

    // common for all methods

    describe('by default', function(){

      describe('URLs', function(){

        var url = require('url');

        afterEach(function(){
          spy.restore();
        })

        it('should add http:// to request URL if missing', function(){
          spy = sinon.spy(needle, 'send_request');
          needle.get('www.google.com');
          spy.args.should.equal('http://www.google.com');
          spy.restore();
        })

        it('should not verify if host is valid', function(){
          spy = sinon.spy(needle, 'send_request');
          needle.get('x1x1x1');
          // console.log(spy);
          spy.args.should.equal('http://x1x1x1');
        })

      })

      describe('headers', function(){

        it('should set Host, Connection, User-Agent and Accept headers', function(){
          var req = needle.get('some.com/url');
          Object.keys(req._headers).sort().join(' ').should.equal('accept connection host user-agent');
        })

        it('should not ask for a deflated response', function(){
          // spy = sinon.spy(needle, 'send_request');
          needle.get('google.com');
          // spy.calledWith.should.not.include( headers['Accept-Encoding'] );
          // spy.restore();
        })

      })

      describe('other options', function(){

        /* TODO
              it('should set a 10 sec timeout timer', function(){
                spyOn(this, 'setTimeout');
                needle.get('server.com/with/some/url')
                expect(setTimeout).toHaveBeenCalledWith(function(){}, 10000);
              })
        */

      })

    })

    describe('when proxied', function(){

      beforeEach(function(){
        req = needle.post('finalhost.com/endpoint', { proxy: 'http://localhost:5141' });
      })

      it('should set Host header to final host', function(){
        req._headers['host'].should.equal('finalhost.com');
      })

      it('should set the original (full) URL as the request path', function(){
        req.path.should.equal('/endpoint');
      })

    })

    // common stuff for all requests

    describe('headers', function(){

      it('should accept setting custom headers', function(){
        var req = needle.get('random.url', { headers: {'X-Custom-Header': '12345-something'} });
        req._headers['x-custom-header'].should.equal('12345-something');
      })

      describe('HTTP basic auth', function(){

        it('should set an Authorization header when username & password are present', function(){

        })

        it('should not set Authorization header if only username or password are set', function(){

        })

        it('Authorization header should be Base64 encoded (& decodeable)', function(){

        })

      })

    })

  })

});
